(* $Id: main.ml 711 2007-07-30 22:37:57Z gerd $
 * ----------------------------------------------------------------------
 * Copyright 2003 by Gerd Stolpmann.
 *
 * This file is part of godi-tools.
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
 *)

open Godi_types;;
open Godi_query;;
open Printf;;

let prestart interactive =
  (* Especially for Cygwin: Complain if PATH contains spaces *)
  ( let path_sep =
      match Sys.os_type with
	  "Win32" -> ";"
	| _ -> ":" in
    let path_str = ( try Sys.getenv "PATH" with Not_found -> "" ) in
    let path = Str.split (Str.regexp path_sep) path_str in
    let newpath =
      List.filter
	(fun p ->
	   not(String.contains p ' ')
	)
	path
    in
    if newpath <> path then (
      eprintf "The PATH variable contains spaces. This is not supported by\n";
      eprintf "this program. The problematic PATH components are removed\n";
      eprintf "for the run time of this program. Beware that this might cause\n";
      eprintf "other problems.\n";
      flush stderr;
      if interactive then (
	printf "Press RETURN to continue... ";
	flush stdout;
	ignore(read_line());
      );
      Godi_exec.set_path (String.concat path_sep newpath)
    )
  );

  (* Unset some findlib-related variables *)
  ( Godi_exec.set_variable "OCAMLFIND_DESTDIR" "";
    Godi_exec.set_variable "OCAMLFIND_METADIR" "";
    Godi_exec.set_variable "OCAMLPATH" "";
    Godi_exec.set_variable "OCAMLFIND_COMMANDS" "";
    Godi_exec.set_variable "OCAMLFIND_LDCONF" "";
  );

  (* Announce special features *)
  Godi_exec.set_variable "GODI_CONSOLE_SUPPORTS_CONFLICTS" "yes";

  let enable_experimental =
    try Sys.getenv "GODI_EXP" <> "" with Not_found -> false in
  Godi_global.experimental_make := enable_experimental;
  
  if enable_experimental then (
    eprintf "Experimental mode is enabled. Not all functions will work\n";
    eprintf "as expected.\n";
    flush stderr;
    if interactive then (
      printf "Press RETURN to continue... ";
      flush stdout;
      ignore(read_line());
    );
  )
;;


let run_interactively() =
  if Array.length Sys.argv > 1 then Arg.current := 1;
  let cont_inst = ref false in
  Arg.parse
    [ "-continue-installation", Arg.Set cont_inst,
      "(internally used)";
    ]
    (fun _ -> raise(Arg.Bad "Unexpected argument"))
    "usage: godi_console interactive <options>";
  
  let t0 = Unix.gettimeofday() in
  let print_times =
    ( try
	let script = Sys.getenv "GODI_TEST_SCRIPT" in
	Godi_tty.load_script script;
	true
      with
	  Not_found -> false
    ) in
  let pg = new Miniui_tty.tty_page() in
  
  if !cont_inst then (
    (* Continue interrupted installation: A previous process was exited
     * because of a Restart exception. The process did install packages.
     * Now resume the installation!
     *)
    pg # put_line "Resuming installation...";
    pg # update();
    let con = Menu_aux.trivial_console pg in
    let src_pkgs = scan_source_packages con in
    let inst_pkgs = get_installed_packages con in
    let persistent_wl = Godi_wishes.read_wishes ~pref_user_error:true () in
    Godi_wishes.apply_wishes_to_packages src_pkgs persistent_wl;
    Godi_wishes.apply_wishes_to_packages inst_pkgs persistent_wl;
    Menu_install.forget_src_pkgs_on_quit := false;
    Menu_install.execute pg src_pkgs inst_pkgs;
    if !Menu_install.forget_src_pkgs_on_quit then
      Godi_query.forget_source_packages();
    (* After that, simply start the normal main menu. *)
  );

  Menu_main.display pg;
  let t1 = Unix.gettimeofday() in
  let pt = Unix.times() in
  if print_times then (
    print_newline();
    printf "Wallclock time:       %5.2f\n" (t1-.t0);
    printf "Process time (user):  %5.2f\n" pt.Unix.tms_utime;
    printf "Process time (sys):   %5.2f\n" pt.Unix.tms_stime;
    printf "Children time (user): %5.2f\n" pt.Unix.tms_cutime;
    printf "Children time (sys):  %5.2f\n" pt.Unix.tms_cstime;
  )
;;


try
  Godi_wget_handler.init();
  Arg.current := 0;
  let f, is_interactive =
    match Filename.basename Sys.argv.(0) with
      | "godi_update" ->
	  Cmd.catch_error Cmd.cmd_update, false
      | "godi_wish" ->
	  Cmd.catch_error Cmd.cmd_wish, false
      | "godi_perform" ->
	  Cmd.catch_error Cmd.cmd_perform, false
      | "godi_list" ->
	  Cmd.catch_error Cmd.cmd_list, false
      | "godi_info" ->
	  Cmd.catch_error Cmd.cmd_info, false
      | "godi_dep" ->
	  Cmd.catch_error Cmd.cmd_dep, false
      | "godi_delete" ->
	  Cmd.catch_error Cmd.cmd_delete, false
      | "godi_add" ->
	  Cmd.catch_error Cmd.cmd_add, false
      | "godi_ptool" ->
	  Cmd.catch_error Cmd.cmd_ptool, false
      | "godi_make" | "make" ->
	  (fun () -> Cmd.catch_error Cmd.cmd_make 1), false
      | _ ->
	  if Array.length Sys.argv = 1 || Sys.argv.(1) = "interactive" then
	    run_interactively, true
	  else
	    Cmd.exec_cmd, false in
  prestart is_interactive;
  f();
  Godi_file.release_all_tmpdirs();
with
    Sys.Break ->
      Godi_file.release_all_tmpdirs();
      exit 10
  | Cmd.Exit_code n ->
      Godi_file.release_all_tmpdirs();
      exit n
  | Restart sh_cmd ->
      Godi_file.release_all_tmpdirs();
      Godi_file.execute ~setx:true sh_cmd
;;

